import os, git, subprocess

'''A VCS must have an "update" method with an optional "force" parameter.'''

# Fetch updates from git
class Git:
	def __init__(self, folder, url, commit):
		self.folder = os.path.abspath(folder)
		self.url = url
		self.commit = commit

	class _ProgressPrinter(git.remote.RemoteProgress):
		def update(self, op_code, cur_count, max_count=None, message=''):
			print self._cur_line+(" "*30)+"\r",

	def update(self):
		isBranch = (self.commit.startswith('origin/'))
		if isBranch:
			branchname = self.commit[len('origin/'):]
		else:
			branchname = "tag"
		# get us a git repository, and the "origin" remote
		if os.path.exists(self.folder):
			# load existing repo
			repo = git.Repo(self.folder)
			origin = repo.remotes.origin
			origin.config_writer.set_value("url", self.url) # make sure we use the current URL
		else:
			# create a new one
			os.makedirs(self.folder)
			repo = git.Repo.init(self.folder)
			origin = repo.create_remote('origin', self.url)
		origin.fetch(progress=Git._ProgressPrinter()) # download new data
		print " "*80+"\r", # clean the line we are in
		# create/find correct branch
		if branchname in repo.heads:
			branch = repo.heads[branchname]
		else:
			branch = repo.create_head(branchname, self.commit)
			if isBranch: # track remote branch
				branch.set_tracking_branch(origin.refs[branchname])
		# update it to the latest remote commit
		branch.checkout()
		repo.git.rebase(self.commit)
		print "...done",
		if repo.head.reference.commit != repo.commit(self.commit):
			print "(keeping local patches around)",
		print

	def version(self):
		repo = git.Repo(self.folder)
		v = repo.git.describe()
		if v.startswith('v'): v = v[1:]
		return v

# Fetch updates via SVN
class SVN:
	def __init__(self, folder, url, versionName):
		self.folder = os.path.abspath(folder)
		self.url = url
		self.versionName = versionName

	def update(self):
		if os.path.exists(self.folder):
			os.chdir(self.folder) # go into repository
			subprocess.check_call(['svn', 'switch', self.url]) # and update to the URL we got
		else:
			subprocess.check_call(['svn', 'co', self.url, self.folder]) # just download it

	def version(self):
		return self.versionName
